/* win.c --- a very basic Xlib application,    */
/* Jeff Pitchers, LUT 1993                     */
/* Extraido de Internet, documentacin de X11  */
/* Set bsico de ficheros de cabecera para X11 */
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>
#include <X11/keysym.h>

/* Variables del programa: */
/* display: Display del servidor al que conectar */
/* screen: N de pantalla a usar en ese display */
/* win: La nica ventana de la aplicacin */
/* event: La estructura de eventos */
/* gc1, gc2: Contextos de grficos */
/* gs1, gs2: Valores de los GCs */
/* message: Mensaje a imprimir en la ventana */
Display *display;
int screen;  
Window win;
XEvent event;      
GC gc1, gc2;       
XGCValues gs1,gs2; 
char *message;     
static char progDefaultMessage[] = {"A Simple Xlib Program"};

/* Funcin principal main, como en todo programa C */
main(argc, argv)
int argc; char **argv;
{

  /* Tratando de conectar al servidor X */
  if ((display = XOpenDisplay(NULL)) == NULL)
  {
    perror("Can't connect to server");
    exit(1);
  }
  screen = DefaultScreen(display);

  /* Crear una ventana */
  win = XCreateSimpleWindow(display,
          RootWindow(display, screen),
          100, 200, 400, 300, 1,
          BlackPixel(display, screen),
          WhitePixel(display, screen));

  /* Seleccionar qu eventos se desea que la aplicacin gestione */
  /* Se selecciona Redibujado + Pulsacin de botn */
  XSelectInput(display, win, ExposureMask | ButtonPressMask);

  /* Iniciar contextos graficos */
  setUpGCs();

 /* Tratar de obtener un mensaje de la fuente de */ 
 /* recursos definidos por el usuario. Esto es similar */
 /* a los ficheros de recursos de Windows */
 /* (en los que se bas luego dicho S.O.), y mediante */
 /* XgetDefault es posible leer del fichero .Xresources */
 /* para obtener valores por defecto para determinadas variables. */
  if ((message = XGetDefault(display, "win", "message")) == NULL)
    message = progDefaultMessage;

  /* Mapear la ventana para que sea visible. */
  XMapWindow(display, win);

  /* Bucle de mensajes. Este bucle es el que le da sentido */
  /* a la aplicacin, y consiste en la captura de los mensajes */
  /* o eventos enviados por el servidor y actuacin en consecuencia */
  /* con ellos. Esto ser explicado en la prxima entrega */
  while (1) {

    /* Coger el evento de la cola */
    XNextEvent(display, &event);
    /* dependiendo del tipo de evento... */
    switch (event.type) {
      /* si nos piden redibujado, redibujar */
      case Expose:
            drawInToWindow(); break;
      /* si se pulsa un botn, salir */
      case ButtonPress:
            exit(0); break;
      /* resto de eventos: nada */
      default: break;
    }
  }
}

/* Funcin que inicializa algunos contextos grficos para */
/* ser utilizados en el programa */
setUpGCs()
/* gc1 se usa para el mensage y rectangulo */
/* gc2 se usa para la ellipse */
{
/* Definir un bitmap para rellenar la elipse */
#define stipple_width 3
#define stipple_height 3
  static char stipple_bits[] = {0x02, 0x05, 0x02};
  Pixmap stipple;

  if ((stipple = XCreateBitmapFromData(display, RootWindow(display, screen),
    stipple_bits, stipple_width, stipple_height)) == 0) {
    perror("Can't create bitmap");
    exit(1);
  }

  gc1 = XCreateGC(display, win, 0, &gs1); /* 0 -> use defaults */
  gc2 = XCreateGC(display, win, 0, &gs2); /* 0 -> use defaults */
  XSetForeground(display, gc1, BlackPixel(display, screen));
  XSetBackground(display, gc1, WhitePixel(display, screen));
  XSetLineAttributes(display, gc1, 2, LineSolid, CapRound, JoinRound);
  XSetStipple(display, gc2, stipple);
  XSetFillStyle(display, gc2, FillOpaqueStippled);
}

/* Funcin que dibuja en la ventana */
drawInToWindow()
{
  /* Imprimir mensaje y dibujar rectngulo */ 
  /* Dibujar elipse rellena definida en el GC */
  XDrawString(display, win, gc1, 130,
              20, message, strlen(message));
  XDrawRectangle(display, win, gc1,
              40, 40, 320, 220);
  XFillArc(display, win, gc2,
              50, 50, 300, 200, 0, 360*64);
}
/* Fin del programa */
